@@ -1,4 +1,3 @@
-
 -- |
 -- The contents of this module can change at any time without warning.
 module System.Log.FastLogger.Internal
@@ -6,9 +5,11 @@ module System.Log.FastLogger.Internal
   , module System.Log.FastLogger.FileIO
   , module System.Log.FastLogger.LogStr
   , module System.Log.FastLogger.Logger
+  , module System.Log.FastLogger.LoggerSet
   ) where
 
 import System.Log.FastLogger.IO
 import System.Log.FastLogger.FileIO
 import System.Log.FastLogger.LogStr
 import System.Log.FastLogger.Logger
+import System.Log.FastLogger.LoggerSet
