@@ -17,6 +17,7 @@ import Data.ByteString.Builder (Builder)
 import qualified Data.ByteString.Builder as B
 import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy as BL
+import qualified Data.ByteString.Short as SBS
 #if MIN_VERSION_base(4,9,0)
 import qualified Data.Semigroup as Semi (Semigroup(..))
 #endif
@@ -47,14 +48,17 @@ data LogStr = LogStr !Int Builder
 
 #if MIN_VERSION_base(4,9,0)
 instance Semi.Semigroup LogStr where
+    {-# INLINE (<>) #-}
     LogStr s1 b1 <> LogStr s2 b2 = LogStr (s1 + s2) (b1 <> b2)
 #endif
 
 instance Monoid LogStr where
     mempty = LogStr 0 (toBuilder BS.empty)
+    {-# INLINE mappend #-}
     LogStr s1 b1 `mappend` LogStr s2 b2 = LogStr (s1 + s2) (b1 <> b2)
 
 instance IsString LogStr where
+    {-# INLINE fromString #-}
     fromString = toLogStr . TL.pack
 
 -- | Types that can be converted to a 'LogStr'. Instances for
@@ -64,60 +68,83 @@ class ToLogStr msg where
     toLogStr :: msg -> LogStr
 
 instance ToLogStr LogStr where
+    {-# INLINE toLogStr #-}
     toLogStr = id
 instance ToLogStr S8.ByteString where
+    {-# INLINE toLogStr #-}
     toLogStr bs = LogStr (BS.length bs) (toBuilder bs)
 instance ToLogStr BL.ByteString where
+    {-# INLINE toLogStr #-}
     toLogStr b = LogStr (fromIntegral (BL.length b)) (B.lazyByteString b)
 instance ToLogStr Builder where
+    {-# INLINE toLogStr #-}
     toLogStr x = let b = B.toLazyByteString x in LogStr (fromIntegral (BL.length b)) (B.lazyByteString b)
+instance ToLogStr SBS.ShortByteString where
+    {-# INLINE toLogStr #-}
+    toLogStr b = LogStr (SBS.length b) (B.shortByteString b)
 instance ToLogStr String where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . TL.pack
 instance ToLogStr T.Text where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . T.encodeUtf8
 instance ToLogStr TL.Text where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . TL.encodeUtf8
 
 -- | @since 2.4.14
 instance ToLogStr Int where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.intDec
 -- | @since 2.4.14
 instance ToLogStr Int8 where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.int8Dec
 -- | @since 2.4.14
 instance ToLogStr Int16 where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.int16Dec
 -- | @since 2.4.14
 instance ToLogStr Int32 where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.int32Dec
 -- | @since 2.4.14
 instance ToLogStr Int64 where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.int64Dec
 
 -- | @since 2.4.14
 instance ToLogStr Word where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.wordDec
 -- | @since 2.4.14
 instance ToLogStr Word8 where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.word8Dec
 -- | @since 2.4.14
 instance ToLogStr Word16 where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.word16Dec
 -- | @since 2.4.14
 instance ToLogStr Word32 where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.word32Dec
 -- | @since 2.4.14
 instance ToLogStr Word64 where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.word64Dec
 
 -- | @since 2.4.14
 instance ToLogStr Integer where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.integerDec
 -- | @since 2.4.14
 instance ToLogStr Float where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.floatDec
 -- | @since 2.4.14
 instance ToLogStr Double where
+    {-# INLINE toLogStr #-}
     toLogStr = toLogStr . B.doubleDec
 
 instance Show LogStr where
